function [e,rec_e,sparse_e,diff_e,W]=Wsparsefun(X,A,S,W,sparse_c,diff_c,diff_pow,reweight)

%Evaluate the Euclidean reconstruction error term between the observation
%matrix X and the model A * S weighted by matrix W, and sparseness and
%temporal continuity terms.
%
%Usage: [e,rec_e,sparse_e,diff_e,W]=
%       Wsparsefun(X,A,S,W,sparse_c,diff_c,diff_pow,reweight)
%
%Input:
%
%X      observation matrix
%A      gain matrix
%S      source matrix
%W      weighted matrix for the euclidean distance
%sparse_c  multiplier of the sparseness cost
%diff_c    multiplier of the temporal continuity cost
%diff_pow  power of temporal continuity term
%reweight (optional) reweight parameter for updating the weights,
%         possible values: 'absmagn', 'sqmagn', 'klmagn','logmagn', 'sndmax'
%
%Output:
%
%e        total error
%rec_e    reconstruction error
%sparse_e sparseness term
%diff_e   temporal continuity term
%W        updated weight matrix

%Tuomas Virtanen 2005

A(A<0)=0;

if ~any(A,1)
  e=Inf;
end

T=size(A,1);
F=size(S,2);

norm_c=sqrt(sum(A.^2,1));
A=A./repmat(norm_c,T,1);
S=S.*repmat(norm_c',1,F);

V=A*S;

D=(X-V).^2;

rec_e=0.5*sum(sum(D.*W));

sparse_e=sparse_c*sum(sum(A));
diff_e=diff_c*sum(sum(abs(diff(A,[],1)).^diff_pow));

e=sparse_e+diff_e+rec_e;



if exist('reweight','var')&&any(ischar(reweight))
  %update W
  if strcmp(reweight,'absmagn')
    D_new=abs(sqrt(X)-sqrt(V));
  elseif strcmp(reweight,'sqmagn')    
    D_new=(sqrt(X)-sqrt(V)).^2;
  elseif strcmp(reweight,'klmagn')
    notvalid=(V==0|X==0);
    V(notvalid)=1;
    X(notvalid)=1;
    D_new=sqrt(X).*log(sqrt(X)./sqrt(V))-sqrt(X)+sqrt(V);
  elseif strcmp(reweight,'logmagn')
    notvalid=(V==0|X==0);
    V(notvalid)=1;
    X(notvalid)=1;
    X=X+median(X(:))/1000;
    V=V+median(V(:))/1000;
    D_new=abs(log(X)-log(V));
  elseif strcmp(reweight,'snrmax')
    R=X-V;
    W=weightest(R);

  end
  

  if ~strcmp(reweight,'snrmax')
    %epsilon1=max(max(D_new))/100;
    %epsilon2=max(max(D))/100;
    epsilon1=median(D_new(:))/10;
    epsilon2=median(D(:))/10;

    W=(D_new+epsilon1)./(D+epsilon2);
  end
  
end
