function [cost,recerror,couplings,priorterm]=gammacost(yepsilon,a,T,z,p,ApriorA,ApriorB,TpriorA,TpriorB)

%Evaluates the cost term between the observation matrix yepsilon and 
%the model T * a, assuming Poisson observation model. Calculates also
%the cost of the Gamma chain of the gains a and auxiliary variables z,
%which are coupled according to the parameter c. Also includes the cost
%caused by the priors of T and a.
%
%Usage: [cost,recerror,couplings,priorterm]=
%       gammacost(yepsilon,a,T,z,p,ApriorA,ApriorB,TpriorA,TpriorB)
%
%Input:
%
%yepsilon    observation matrix
%a           gain matrix
%T           template matrix
%z           auxiliary variable matrix
%p           coupling parameter between gains and auxiliary variables
%optional input parameters:
%ApriorA     alpha parameter matrix of the Gamma prior distribution
%            of the gains
%ApriorB     beta parameter matrix  of the Gamma prior distribution
%            of the gains
%TpriorA     alpha parameter matrix of the Gamma prior of the templates
%TpriorB      beta parameter matrix  of the Gamma prior of the templates
%
%Output:
%
%cost        total cost
%recerror    reconstruction error cost
%couplings   Gamma chain cost
%priorterm   prior costs
%
%See T. Virtanen, A. T. Cemgil, and S. J. Godsill. "Bayesian Extensions
%to Non-negative Matrix Factorisation for Audio Signal Modelling,"
%ICASSP 2008 for explanation of the terms.

%Tuomas Virtanen 2007


%calculate the gamma modeling error 

%ez=(2*p-1)/2;
ez=p;

if ~exist('ApriorA','var')
  priorA=0;
  priorB=0;
end

model=T*a;

notvalid=(yepsilon==0 | model==0);
yepsilon(notvalid)=1;
model(notvalid)=1;

couplingmodel1=p*z.*a;
couplingmodel2=p*z(:,[end, 1:end-1]).*a;
recerror=sum(sum(model+yepsilon.*log(yepsilon./model)-yepsilon));
a(a==0)=1;
Apriorterm=-sum(sum(ApriorA.*log(a)-a.*ApriorB));


if exist('TpriorA','var')&&~isempty(TpriorA)
  T(T==0)=1;
  Tpriorterm=-sum(sum(TpriorA.*log(T)-T.*TpriorB));
else
  Tpriorterm=0;
end

if p~=0
    coupling1=sum(sum(couplingmodel1+ez*log(ez./couplingmodel1)-ez));
    coupling2=sum(sum(couplingmodel2+ez*log(ez./couplingmodel2)-ez));
    couplings=coupling1+coupling2;
    cost=recerror+couplings;
else
    cost=recerror;
    couplings=0;
end

priorterm=Apriorterm+Tpriorterm;
cost=cost+priorterm;


