function [obj,W]=klfun(X,A,S,W,threshold,reweight,priorA,priorB)

%Evaluate the weighted cost term between the observation matrix X
%and the model A * X, assuming probabilistic Poisson model
%(the cost is equivalent to the generalized kullback-leibler divergence).
%
%Includes also the cost caused by the priors of S, if they are provided
%
%Usage: [obj,W]=klfun(X,A,S,W,threshold,reweight,priorA,priorB)
%
%Input:
%
%X          observation matrix
%A          gain matrix
%S          source matrix
%threshold  fixed additive term
%reweight (optional) updates the weight matrix according to the
%           specified algorithm
%           possible values: 'absmagn', 'sqmagn', 'klmagn', 'logmagn'
%priorA (optional) alpha parameter of the Gamma prior distribution of S
%           (the size should be the size of S) 
%priorB (optional)    beta parameter of the Gamma prior distribution of S
%

%Tuomas Virtanen 2007


V=A*S;
notvalid=(V==0|X==0);
V(notvalid)=1;
X(notvalid)=1;

D=(X+threshold).*log((X+threshold)./(V+threshold))-X+V;
obj=sum(sum(D.*W));

if exist('priorA','var') && ~isempty(priorA)
  notvalidS=(S==0 | priorB==0);
  S(notvalidS)=1;
  normalization=priorA.*log(priorA./priorB)-priorA;
  normalization(isnan(normalization))=0;
  priorD=-priorA.*log(S)+S.*priorB+normalization;
  obj=obj+sum(priorD(:));
end


if exist('reweight','var')&&any(ischar(reweight))
  %update W
  if strcmp(reweight,'absmagn')
    D_new=abs(sqrt(X)-sqrt(V));
  elseif strcmp(reweight,'sqmagn')    
    D_new=(sqrt(X)-sqrt(V)).^2;
  elseif strcmp(reweight,'klmagn')
    %D_new=sqrt(X).*log(sqrt(X)./sqrt(V))-sqrt(X)+sqrt(V);
    alfa=0.3;
    new_X=(X.^alfa+threshold.^alfa);
    new_V=(V.^alfa+threshold.^alfa);
    D_new=new_X.*log(new_X./new_V)-new_X+new_V;
  elseif strcmp(reweight,'logmagn')
    X=X+median(X(:))/1000;
    V=V+median(V(:))/1000;
    D_new=abs(log(X)-log(V));
  end

  neg=((X-V)<0);
  D_new(neg)=D_new(neg)*2.5;

  
  D_new(notvalid)=1;
  D(notvalid)=1;

  epsilon1=median(D_new(:))/10;
  epsilon2=median(D(:))/10;
  W=(D_new+epsilon1)./(D+epsilon2);
    
  W(notvalid)=mean(mean(W));

end
