function A=multiupdate(X,A,S,rec_objective,sparse_c,diff_c,threshold,priorA,priorB)

%Update gain matrix A so that the value of the given cost function decreases.
%The cost function includes the recostruction error term f(X-A*S) and 
%sparseness and temporal continuity terms, and a prior term. This function
%implements various cost functions f.
%
%Usage: A=multiupdate(X,A,S,rec_objective,sparse_c,...
%                     diff_c,threshold,priorA,priorB)
%Input:
%
%X     observation matrix
%A     gain matrix to be updated
%S     source matrix
%rec_objective reconstruction error function, possible values:
%      'euclidean'  Gaussian model, factorial model for the mean
%      'divergence' Poisson model
%      'log-divergence' Gaussian model, factorial model for the variance
%      'invgamma'   inverse Gamma model
%sparse_c multiplier of the sparseness term
%diff_c   multiplier of the temporal continuity term
%threshold additive term in the Poisson model
%priorA (optional) alpha paramter in the Gamma prior of A
%priorB (optional) beta parameter in the Gamma prior of A
%
%Output:
%
%A     updated gain matrix
%
%Different models are described in T. Virtanen and A. T. Cemgil. "Bayesian
%extensions to Non-negative Matrix Factorisation using Gamma Chain Priors."
%Technical report, Cambridge University, October 2007.
%

%Tuomas Virtanen 2007


if strcmp(rec_objective,'euclidean')
  multi=X*S';
  divis=A*S*S';
elseif strcmp(rec_objective,'divergence')
  Y=A*S+threshold;
  notvalid=~Y;
  Y(notvalid)=1;
  X(notvalid)=1;
  multi=(X./Y)*S';
  divis=ones(size(X))*S';
  notvalid=isnan(multi);
  if exist('priorA','var') && ~isempty(priorA)
    multi=multi+priorA./(A+1e-10);
    divis=divis+priorB;
  end
elseif strcmp(rec_objective,'log-divergence')
  V=A*S;
  notvalid=~V;
  V(notvalid)=1;
  X(notvalid)=1;
  multi=(X./V.^2)*S';
  divis=(ones(size(X))./V)*S';
elseif strcmp(rec_objective,'invgamma')
  Y=A*S;
  notvalid=(~Y | ~X);
  Y(notvalid)=1;
  X(notvalid)=1;
  multi=(1./Y)*S';
  divis=(1./X)*S'; 
else
  error(['recfun ',recfun,' not implemented'])
end


T=size(A,1);

if sparse_c
  norm_A=1./sqrt(sum(A.^2,1));
  divis=divis+repmat(sparse_c*norm_A,T,1);
  multi=multi+sparse_c*A.*repmat(sum(A,1),T,1).*repmat(norm_A.^3,T,1);
end


if diff_c
  norm_A=1./sum(A.^2,1);
  diff_A=diff(A,[],1).^2;
  
  AA=4*A;
  AA([1 end],:)=AA([1 end],:)/2;
  %only power 2 implemented currently
  if 1
    
    %positive gradient: divisor, negative gradient: multiplicator
%    divis=divis+AA.*repmat(diff_c*norm_A,T,1);
%    multi(1:end-1,:)=multi(1:end-1,:)+2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
%    multi(2:end,:)=multi(2:end,:)+2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
%    multi=multi+4*A.*repmat(diff_c*sum(diff_A,1).*norm_A.^2,T,1);

%    divis=divis+4*A.*repmat(diff_c*norm_A,T,1);
%    multi(1:end-1,:)=multi(1:end-1,:)+2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
%    multi(2:end,:)=multi(2:end,:)+2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
%    multi=multi+2*A.*repmat(diff_c*sum(diff_A,1).*norm_A.^2,T,1);


%BB=sum(A(1:end-1,:).*A(2:end,:),1);
%    divis=divis+4*A.*repmat(diff_c*norm_A,T,1);
%    divis=divis+4*A.*repmat(diff_c*norm_A.^2.*BB,T,1);
%    multi(1:end-1,:)=multi(1:end-1,:)+2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
%    multi(2:end,:)=multi(2:end,:)+2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
%CC=sum(A(1:end-1,:).^2,1)+sum(A(2:end,:).^2,1);
%    multi=multi+2*A.*repmat(diff_c*norm_A.^2.*CC,T,1);


%    divis=divis+AA.*repmat(diff_c*norm_A,T,1);
%    BB=sum(A(1:end-1,:).*A(2:end,:),1);
%    divis=divis+4*A.*repmat(diff_c*norm_A.^2.*BB,T,1);
%    multi(1:end-1,:)=multi(1:end-1,:)+2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
%    multi(2:end,:)=multi(2:end,:)+2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
%    CC=sum(A(1:end-1,:).^2,1)+sum(A(2:end,:).^2,1);
%    multi=multi+2*A.*repmat(diff_c*norm_A.^2.*CC,T,1);
    

    divis=divis+AA.*repmat(diff_c*norm_A,T,1);
    multi(1:end-1,:)=multi(1:end-1,:)+2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
    multi(2:end,:)=multi(2:end,:)+2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
    multi=multi+2*A.*repmat(diff_c*sum(diff_A,1).*norm_A.^2,T,1);


  else
    %positive&negative gradient: divisor (tested, does not work!)
    divis=divis+AA.*repmat(diff_c*norm_A,T,1);
    divis(1:end-1,:)=divis(1:end-1,:)-2*A(2:end,:).*repmat(diff_c*norm_A,T-1,1);
    divis(2:end,:)=divis(2:end,:)-2*A(1:end-1,:).*repmat(diff_c*norm_A,T-1,1);
    divis=divis-4*A.*repmat(diff_c*sum(diff_A,1).*norm_A.^2,T,1);  
  end
  
end

notvalid=(~multi|~divis);
multi(notvalid)=1;
divis(notvalid)=1;
A=A.*multi./divis;



%set values which are almost zero to zero
A(A<1e-16)=0;
