function [A,S,obj,iter]=nmffactorize(X,params,S)
% NMFFACTORIZE factorizes non-negative matrix X into a product A*S
% of non-negative matrices A and S.
%
% SYNTAX:
% [A,S]=nmffactorize(X,params)
%
% INPUT:    
% X        matrix X to be factorized (frames x frequencies)
% params   pareters to control the factorization. The fields are explained
%          below
% S (optional) fixed S, or prior for S
%
% sources   number of sources
%
% rec_objective  distance measure for the reconstruction error, either
%               'euclidean', 'divergence', 'log-divergence', or 'invgamma'
%
% optional fields:
%
% algorithm      algorithm used to update A, either
%                'multiplicative' or 'gradient'
%
% basisalgorithm algorithm ued to update S, either 'multiplicative',
%                'gradient' or 'fixed' (does not update S)
%
% sparse_c       weight for the sparseness cost (default=0)
% diff_c         weight for the temporal continuity cost (default=0)
% diff_pow       power for the temporal continuity cost,
%                either 1 or 2, (normal), -1 (gamma)
%
% priorcontribution parameter which controls the contribution of the
%                   prior of S (extreme values: 0=flat prior, Inf=fixed S)
%
% stop_c         stopping criterion (default=1e-5)
% stop_n         stopping critetion (default=29)
%
% stepsize       initial stepsize in the gradient algorithm (default 0.001)
% momentgain     weight for the moment term in the gradient alg (default 0.7)
%
% printing       flag: 1=print the value of the cost function
%                at each iteration (default=0)
% plotA          flag: 1=plot A at eat iteration (default=0)
% normalize      fix the norm of the gains to unity (default=1)
%
% OUTPUT:
%
% A         mixing matrix (frames x sources)
% S         source matrix (sources x frequencies)
%
% See the following publications for the explanation of the 
% models and algorithms:
%
% T. Virtanen. "Sound Source Separation in Monaural Music Signals",
% PhD Thesis, Tampere University of Technology, 2007
%
% T. Virtanen and A. T. Cemgil. "Bayesian extensions to Non-negative
% Matrix Factorisation using Gamma Chain Priors."
% Technical report, Cambridge University, October 2007.
%

% Tuomas Virtanen 2004-2007


if isfield(params,'sparse_c')
  sparse_c=params.sparse_c;
else
  sparse_c=0;
end

if isfield(params,'diff_c')
  diff_c=params.diff_c;
else
  diff_c=0;
end

if isfield(params,'stop_c')
  stop_c=params.stop_c;
else
  stop_c=1e-5;
end

if isfield(params,'stop_n')
  stop_n=params.stop_n;
else
  stop_n=29;
end

if isfield(params,'diff_pow')
  diff_pow=params.diff_pow;
else
  diff_pow=2;
end

if isfield(params,'aud_pow')
  aud_pow=params.aud_pow;
else
  aud_pow=1;
end



if isfield(params,'threshold')
  threshold=params.threshold;
else
  threshold=0;
end

if ~isfield(params,'algorithm')
  params.algorithm='multiplicative';
end

if ~isfield(params,'basisalgorithm')
  params.basisalgorithm='multiplicative';
end


if ~isfield(params,'plotA')
params.plotA=0;
end

if ~isfield(params,'printing')
params.printing=0;
end

W=ones(size(X));

if ~strcmp(params.rec_objective,'log-divergence')
  scale=sqrt(1000)/sqrt(sum(sum((X.*W).^2)));
else
  scale=sqrt(1000)/sum(sum(abs(log(X(X>0)))));
end
X=X*scale;

threshold=threshold*scale;



F = size(X,2);
T = size(X,1);
sources=params.sources;


% Initializing A
A = abs(randn(T,sources));
A = A./repmat(sqrt(sum(A.^2,1)),T,1);


%Prior distributions for the spectra--------------------

priorA=[];
priorB=[];

if ~isfield(params,'priorcontribution')
  params.priorcontribution=0;
end
if (params.priorcontribution<0)
  error('Prior contribution has to be non-negative')
end
if ~isfinite(params.priorcontribution)
  params.basisalgorithm='fixed';
end
if strcmp(params.basisalgorithm,'fixed')
  fixedS=S;
end

if exist('S','var')
  if size(S,2)~=F || size(S,1)>sources
    error('Input S has wrong dimensions')
  end
  randsources=size(S,1)+1:sources;
  if length(randsources)
    S(randsources,:)=abs(randn(length(randsources),F));
    S(randsources,:)=S(randsources,:)./repmat(sqrt(sum(S(randsources,:).^2,2)),1,F);
  end
  
  if ~strcmp(params.basisalgorithm,'fixed') && isfinite(params.priorcontribution) && params.priorcontribution>0
    priorA=params.priorcontribution;
    priorB=params.priorcontribution./S;    
    if length(randsources)    
      priorA=repmat(priorA,sources,F);
      priorA(randsources,:)=0;
      priorB(randsources,:)=0;
    end
  end
else
  % Initializing S
  S = abs(randn(sources,F));
  S = S./repmat(sqrt(sum(S.^2,2)),1,F);
end

%-------------------------------------------------

%initialize auxiliary Gamma variables
if diff_pow==-1
    Z = abs(randn(size(A)));
    Z = Z./repmat(sqrt(sum(Z.^2,1)),T,1);
end


diff_c=diff_c*2/sources/sqrt(T);

sparse_c=sparse_c/sources/sqrt(T);


% Loop indefinitely
iter = 0;

positive_steps=0; %number of steps which have increased the cost function
refobj=Inf;
obj=Inf;

stepsize=0.001;

momentgA=zeros(size(A));
momentgain=0.7;
momentgS=zeros(size(S));

t_A=10;
t_S=10;
decrat=0.5;
incrat=2; %bold driver algorithm for the learning rate adaptation
%incrat=1.02;

mod_X=(X+threshold).^aud_pow;


while 1
  if params.plotA
    if mod(iter,params.plotA)==0
      hold off
      plot(A)
      drawnow;
    end
  end

  if strcmp(params.basisalgorithm,'multiplicative')||strcmp(params.basisalgorithm,'variational')||length(randsources)
    % Update S
    if strcmp(params.rec_objective,'euclidean')
      %update rule for the Eucliden distance
      S=weightedSupdate(A,S,X,W);
    elseif strcmp(params.rec_objective,'divergence')
      %update rule for KL divergence
      if aud_pow==1
        S=multiupdate(X',S',A',params.rec_objective,0,0,threshold,priorA',priorB')';
      else
        S=powklupdate(X,A,S,aud_pow,W,threshold,[],mod_X);
      end
    elseif strcmp(params.rec_objective,'log-divergence')
      %update rule for log-distance
      if strcmp(params.basisalgorithm,'multiplicative')
         S=multiupdate(X',S',A',params.rec_objective,0,0)';
      else
         S=variationalupdate(S',A',X')'; 
      end      
    elseif strcmp(params.rec_objective,'invgamma')
         S=multiupdate(X',S',A',params.rec_objective,0,0)';        
    end    
  elseif strcmp(params.basisalgorithm,'gradient')
    %gradient algorithm for basis functions
    if strcmp(params.rec_objective,'euclidean')
      error('Euclidean gradient update not implemented yet')
    else
      %divergence
      [gS,rec_e]=klgrad(A,S,W,threshold,1,mod_X);
      gS=gS+momentgain*momentgS;
      momentgS=gS;

      %find step size
      newobj=Inf;
      obj=rec_e;
      while (newobj>=obj)&(t_S>1e-12)
        newS=S-t_S*gS;
        newS(newS<0)=0;
        newobj=sparsepowklfun(A,newS,W,0,0,diff_pow,threshold,1,mod_X,1);
        t_S=t_S*decrat;
      end
      t_S=t_S/decrat;
    
      obj=newobj;
      model_e=newobj;

      S=S-t_S*gS;
      t_S=t_S*incrat;

      S(S<0)=0;
    end
  end
  if strcmp(params.basisalgorithm,'fixed')
    %keep given sources fixed
    S(1:size(fixedS,1),:)=fixedS;
  end
  
  

  %update A, gradient algorithm
  if strcmp(params.algorithm,'gradient')
    
    if aud_pow~=1
      error('Gradient algorithm for arbitrary power has not been implemented')
    end
    
    
    if diff_pow==1
      gAdiff=absdiffgrad(A);
    elseif diff_pow==2
      gAdiff=sqdiffgrad(A,diff_c);
    end
  
    gAsparse=sparsegrad(A,sparse_c);

    if strcmp(params.rec_objective,'euclidean')
      dA=eucgrad(X,A,S,W);
    elseif strcmp(params.rec_objective,'divergence')
      dA=klgrad(S',A',W',threshold',1,mod_X')';
    else
      dA=loggrad(A,S,X);
    end
    
    
    gA=dA+diff_c(1)*gAdiff+sparse_c(1)*gAsparse;
  
    gA=gA+momentgain*momentgA;
    momentgA=gA;

    if strcmp(params.rec_objective,'euclidean')
      oldobj=Wsparsefun(X,A,S,W,sparse_c,diff_c,diff_pow);
    elseif strcmp(params.rec_objective,'divergence')
      oldobj=sparsepowklfun(A,S,W,sparse_c,diff_c,diff_pow,threshold,1,mod_X,1);
    else
      oldobj=logcost(X,A,S);
    end
    
    newobj=Inf;
    while (newobj>=oldobj)&(stepsize>1e-7);
      newA=A-stepsize*gA;
      if strcmp(params.rec_objective,'euclidean')
	[newobj,rec_e,sparse_e,diff_e]=Wsparsefun(X,newA,S,W,sparse_c,diff_c,diff_pow);
      elseif strcmp(params.rec_objective,'divergence')
	[newobj,rec_e,sparse_e,diff_e]=sparsepowklfun(newA,S,W,sparse_c,diff_c,diff_pow,threshold,1,mod_X,1);
      else
	newobj=logcost(X,A,S);
      end
      stepsize=stepsize/2;
    end
    stepsize=stepsize*2*1.05;

    A=A-stepsize*gA;

    % Prevent negative values from entering A
    A(A<0)=0;
  
  
  else
    %multiplicative updates
    if diff_pow~=-1
        if strcmp(params.algorithm,'multiplicative')
           A=multiupdate(mod_X,A,S,params.rec_objective,sparse_c,diff_c,threshold);
        else
           A=variationalupdate(A,S,X);
        end
    else
        [A,Z]=gammagainupdate(mod_X',A',S',Z',diff_c,1,sparse_c);
        A=A';
        Z=Z';
    end
    
  end
  
  
  %if diff_pow~=-1 %no normalization for gamma model
  if isfield(params,'normalize')&&params.normalize
    norm_c=sqrt(sum(A.^2,1));
    A=A./repmat(norm_c,size(A,1),1);
    S=S.*repmat(norm_c',1,size(S,2));
    if diff_pow==-1
        Z=Z.*repmat(norm_c,size(Z,1),1);
    end    
  end

  
  %print objective
  if strcmp(params.rec_objective,'euclidean')
    [obj,rec_e,sparse_e,diff_e]=Wsparsefun(X,A,S,W,sparse_c,diff_c,diff_pow);
  elseif strcmp(params.rec_objective,'divergence')
      if diff_pow==-1 %Gamma prior model
          [obj,rec_e,diff_e,sparse_e]=gammacost(mod_X',A',S',Z',diff_c,0,sparse_c,priorA',priorB');
      else
        %KL divergence
        if aud_pow==1
        rec_e=klfun(X,A,S,W,threshold,[],priorA,priorB);
        else
        [tmp,rec_e]=sparsepowklfun(A,S,W,0,0,diff_pow,threshold,aud_pow,mod_X,1);
        end
        sparse_e=sparse_c*sum(sum(A));
        diff_e=diff_c*sum(sum(abs(diff(A,[],1)).^diff_pow));

        obj=rec_e+sparse_e+diff_e;
      end
      
  elseif strcmp(params.rec_objective,'log-divergence')
    obj=logcost(X,A,S);
    rec_e=obj;
    sparse_e=0;
    diff_e=0;
  elseif strcmp(params.rec_objective,'invgamma')
    obj=invgammacost(X,A,S);
    rec_e=obj;
    sparse_e=0;
    diff_e=0;      
  end
  
  
  
  if params.printing
    fprintf('\r[%d]: ', iter);
    %fprintf('[%d]: ', iter);
    fprintf('%.3f',obj);
    fprintf('     Model: %.3f',rec_e);
    fprintf('  Sparse: %.3f',sparse_e);
%    fprintf('  Discontinuity: %.3f\n',diff_e);
    fprintf('  Discontinuity: %.3f\r',diff_e);
  end

  if refobj<obj
  %  format compact
  %  disp('Increased value of the total cost')
  end
  
  
  if refobj<obj*(1+stop_c)
    positive_steps=positive_steps+1;
  end
  if (positive_steps>stop_n)|(obj<0.01)
    if params.printing
      fprintf('\n');
    end
    norm_c=sqrt(sum(A.^2,1));
    A=A./repmat(norm_c,size(A,1),1);
    S=S.*repmat(norm_c',1,size(S,2));

    A=A/scale;    
    return;
  end
  
  refobj=obj;
  
  % Update iteration counter
  iter = iter+1;
        
end
