function [e,rec_e,sparse_e,diff_e]=sparsepowklfun(A,S,W,sparse_c,diff_c,diff_pow,threshold,alfa,mod_X,reweight)

%calculates the weighted divergence between mod_X and A*S+threshold
%and the sparseness and temporal continuity terms
%
%input:   A         gain matrix
%         S         source matrix
%         W         weights for the spectrogram
%         sparse_c  weight for the sparseness term.
%                   if scalar, use sparsefun=abs(x), if vector, use
%                   sparsefun=log(x^2+1)
%                   if sparse_c(2)==1, use sparsefun(x/norm(all_x))
%                   if sparse_c(2)==2, use sparsefun(x/std(x))
%         diff_c    weight for the temporal continuity term
%                   if vector, the second term is the additional weight for
%                   decreasing gain
%         diff_pow  power used in the difference error term
%         threshold threshold term in the divergence
%         alfa      power term in the divergence
%         mod_X     (X+threshold).^alfa, given to decrease the amount
%                   of calculations
%         reweight  weight for negative reconstruction errors
%
%output:  e         total error
%         rec_c     divergence
%         sparse_e  sparseness error term
%         diff_e    temporal continuity error

A(A<0)=0;

if ~any(A,1)
  e=Inf;
end

T=size(A,1);
F=size(S,2);

norm_c=sqrt(sum(A.^2,1));
A=A./repmat(norm_c,T,1);
S=S.*repmat(norm_c',1,F);

if alfa~=1
  V=(A*S+threshold).^alfa;
else
  V=A*S+threshold;
end

notvalid=(mod_X==0)|(V==0);
mod_X(notvalid)=1;
V(notvalid)=1;

D=mod_X.*log(mod_X./V)-mod_X+V;

if reweight~=1
  neg=((mod_X-V)<0);
  W=ones(size(W));
  W(neg)=reweight;
end

rec_e=sum(sum(D.*W));

if length(sparse_c)==1
  %sparsefun=|x|
  sparse_e=sparse_c*sum(sum(A));
else
  %sparsefun=log(x^2+1)
  if sparse_c(2)==1
    sparse_A=log(A.^2+1);
  else
    sparse_A=log(T*A.^2+1);
  end
  sparse_e=sparse_c(1)*sum(sum(sparse_A));
end

if length(diff_c)==1
  diff_e=diff_c*sum(sum(abs(diff(A,[],1)).^diff_pow));
else
  %different weights for positive and negative differences
  positive=(diff(A,1,1)>0);
  diff_W=repmat(diff_c(2),size(positive));
  diff_W(positive)=1/diff_c(2);
  diff_e=diff_c(1)*sum(sum((abs(diff(A,[],1)).^diff_pow).*diff_W));
end

e=sparse_e+diff_e+rec_e;

